@@ -13,6 +13,7 @@ import signal
 import sys
 import time
 import uuid
+import multiprocessing
 
 FRAMES_PATH = os.getenv("FRAMES_PATH", 'frames')
 
@@ -31,7 +32,8 @@ REDIS_TTL = int(os.getenv("REDIS_TTL", 60))
 REDIS_SAMPLE_RATE = int(os.getenv("REDIS_SAMPLE_RATE", 8)) # 1/8 images will be post to redis
 redis_hosts = requests.get(REDIS_HOST_LIST_URL).json() if REDIS_HOST_LIST_URL else [REDIS_HOST]
 
-WORKERS = int(os.getenv("WORKERS", 50))
+WORKERS = int(os.getenv("WORKERS", multiprocessing.cpu_count()))
+EVENT_QUEUE_MAX_SIZE = int(os.getenv("EVENT_QUEUE_MAX_SIZE", WORKERS * 2))
 BASE64_ENCODE = "BASE64_ENCODE" in os.environ
 LOG_FILE = os.getenv("LOG_FILE", None)
 LOG_LEVEL = getattr(logging, os.getenv("LOG_LEVEL", "debug").upper())
@@ -77,13 +79,18 @@ def post(path):
 
 @log_on_error
 def post_http(channel, data, path):
-    for http_host in [h for h in http_hosts if h]:
-        url = HTTP_PUBLISH_URL_TEMPLATE.format(channel=channel, host=http_host, port=HTTP_PORT)
-        r = requests.post(url, data=data, timeout=0.5)
-        if r.status_code == 200:
-            logger.debug('Pushed {} to {}'.format(path, url))
-        else:
-            logger.error(r)
+    for host in [h for h in http_hosts if h]:
+        post_http_to_host(channel, data, path, host)
+
+
+@log_on_error
+def post_http_to_host(channel, data, path, host):
+    url = HTTP_PUBLISH_URL_TEMPLATE.format(channel=channel, host=host, port=HTTP_PORT)
+    r = requests.post(url, data=data, timeout=0.5)
+    if r.status_code == 200:
+        logger.debug('Pushed {} to {}'.format(path, url))
+    else:
+        logger.error(r)
 
 
 @log_on_error
@@ -143,7 +150,8 @@ def run():
     setup_logger()
     logger.info('Started')
     event_handler = EventHandler()
-    observer = Observer()
+    observer = Observer(timeout=0.1)
+    observer.event_queue.maxsize = EVENT_QUEUE_MAX_SIZE
     try:
         delete_all_files(FRAMES_PATH)
         observer.schedule(event_handler, path=FRAMES_PATH, recursive=True)
@@ -154,8 +162,10 @@ def run():
             time.sleep(1)
             now = datetime.datetime.now()
             if now - event_handler.last_event > datetime.timedelta(minutes=1):
+                logger.warning("No events received in the last minute. "
+                               "Unfinished tasks: %d" % observer.event_queue.unfinished_tasks)
+                assert observer.is_alive()
                 event_handler.last_event = now
-                logger.warning("No events received in the last minute.")
     except KeyboardInterrupt:
         logger.warning("Keyboard interruption. Shuting down.")
     except Exception as err:
